#include "chad.h"

CChadGame::CChadGame() {
	m_Ogre = new Root; 
	this->loadConfig();

	if(!m_Ogre->showConfigDialog()) return;

	m_Ogre->initialise(true, "Hanging Chad"); 
	this->initialise();
	this->createScene();
}

CChadGame::~CChadGame() {
	delete m_FrameListener;
	delete m_EventProcessor;
	delete m_Ogre;
}

bool CChadGame::loadConfig() {
	ConfigFile cf;
	cf.load("resources.cfg");

	ConfigFile::SectionIterator seci = cf.getSectionIterator();
	String secName, typeName, archName;

	while (seci.hasMoreElements()) {
		secName = seci.peekNextKey();
		ConfigFile::SettingsMultiMap *settings = seci.getNext();
		ConfigFile::SettingsMultiMap::iterator i;

		for (i = settings->begin(); i != settings->end(); ++i) {
			typeName = i->first;
			archName = i->second;
			ResourceGroupManager::getSingleton().addResourceLocation(archName, typeName, secName);
		}
	}

	return true;
}

void CChadGame::initialise() {
	m_SceneMgr = m_Ogre->createSceneManager(ST_EXTERIOR_CLOSE); 

	m_Camera = m_SceneMgr->createCamera("Player Cam"); 
	m_Camera->setPosition(Vector3(250,70,350));
	m_Camera->setNearClipDistance(1);

	m_Viewport = m_Ogre->getAutoCreatedWindow()->addViewport(m_Camera);
	m_Viewport->setBackgroundColour(ColourValue(0, 0, 0));

	m_Camera->setAspectRatio(Real(m_Viewport->getActualWidth()) / Real(m_Viewport->getActualHeight()));

	m_EventProcessor = new EventProcessor(); 
	m_EventProcessor->initialise(m_Ogre->getAutoCreatedWindow()); 
	m_EventProcessor->startProcessingEvents(); 
	
	m_FrameListener = new CChadFrameListener(m_EventProcessor->getInputReader(), m_Camera, m_SceneMgr);
	m_Ogre->addFrameListener(m_FrameListener);

	m_EventProcessor->addMouseListener(m_FrameListener);
	m_EventProcessor->addMouseMotionListener(m_FrameListener);

	ResourceGroupManager::getSingleton().initialiseAllResourceGroups();
}

int CChadGame::run() {
	m_Ogre->startRendering();
	return 0;
}

void CChadGame::createScene() {
	m_SceneMgr->setWorldGeometry("terrain.cfg");
	m_SceneMgr->setSkyDome(true, "Examples/CloudySky");
}

int main(int argc, char **argv) { 
	CChadGame* game = new CChadGame();
	game->run();
	delete game;

	return 0;
}
